<?php

namespace Tygh\Payments\Processors;

class Bspb
{
    public function initPay($processor_data, $order_info)
    {
        $order_id = $order_info['order_id'];
        $order_sum = (float)$order_info['total'] * 100;

        $xml = new \SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><TKKPG></TKKPG>');
        $xmlRequest = $xml->addChild('Request');
        $xmlRequest->addChild('Operation', 'CreateOrder');
        $xmlRequest->addChild('Language', $processor_data['processor_params']['language']);
        $xmlOrder = $xmlRequest->addChild('Order');
        $xmlOrder->addChild('OrderType', $processor_data['processor_params']['type']);
        $xmlOrder->addChild('Merchant', $processor_data['processor_params']['merchant']);
        $xmlOrder->addChild('Amount', $order_sum);
        $xmlOrder->addChild('Currency', $processor_data['processor_params']['currency']);
        $xmlOrder->addChild('Description', 'Опалат заказа №' . $order_id);

        $resultUrl = $this->getResultUrl();
        $xmlOrder->addChild('ApproveURL', $resultUrl['ApproveURL']);
        $xmlOrder->addChild('CancelURL', $resultUrl['CancelURL']);
        $xmlOrder->addChild('DeclineURL', $resultUrl['DeclineURL']);

        $xmlAddParams = $xmlOrder->addChild('AddParams');
        $xmlAddParams->addChild('ShopOrderNum', $order_id);

        $strResponse = $this->sendPostXml(
            $this->getPayUrl($processor_data['processor_params']['mode']),
            $xml->asXML(),
            $processor_data['processor_params']['rem_file'],
            $processor_data['processor_params']['key_file']);
        if (is_array($strResponse) && strlen($strResponse['error']) == 0) {
            $response = $this->XmlToArray($strResponse['data']);
            if ($response['Response']['Status'] == '00') {
                $urlToBank = $response['Response']['Order']['URL'];
                $params = [
                    'ORDERID' => $response['Response']['Order']['OrderID'],
                    'SESSIONID' => $response['Response']['Order']['SessionID']
                ];
                fn_create_payment_form($urlToBank, $params, 'bspb', true, 'GET');
            } else {
                $pp_response['reason_text'] = 'Error: status ' . $response['Response']['Status'];
            }
        } else {
            $pp_response['reason_text'] = 'Error: ' . $strResponse['error'];
        }

        $pp_response['order_status'] = 'F';
        $pp_response['reason_text'] = print_r($processor_data['processor_params']['type'],true);

        fn_finish_payment($order_id, $pp_response);
        fn_order_placement_routines('route', $order_id, false);
    }

    public function resultPay()
    {
        $data = $this->XmlToArray($_POST['xmlmsg']);
        if (array_key_exists('Message', $data)) {
            $orderId = $data['Message']['OrderID'];
            $sessionId = $data['Message']['SessionID'];
            $order_id = $this->findOrderByDesc($data['Message']['OrderDescription']);
        } else {
            $orderId = $data['OrderID'];
            $sessionId = $data['SessionId'];
            $order_id = $this->findOrderByDesc($data['OrderDescription']);
        }

        $order_info = fn_get_order_info($order_id);
        $processor_data = fn_get_processor_data($order_info['payment_id']);

        $xml = new \SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><TKKPG></TKKPG>');
        $xmlRequest = $xml->addChild('Request');
        $xmlRequest->addChild('Operation', 'GetOrderInformation');
        $xmlRequest->addChild('Language', $processor_data['processor_params']['language']);
        $xmlOrder = $xmlRequest->addChild('Order');
        $xmlOrder->addChild('Merchant', $processor_data['processor_params']['merchant']);
        $xmlOrder->addChild('OrderID', $orderId);
        $xmlRequest->addChild('SessionID', $sessionId);
        $xmlRequest->addChild('ShowParams', 'true');
        $xmlRequest->addChild('ShowOperations', 'true');
        $xmlRequest->addChild('ClassicView', 'true');

        $strResponse = $this->sendPostXml(
            $this->getPayUrl($processor_data['processor_params']['mode']),
            $xml->asXML(),
            $processor_data['processor_params']['rem_file'],
            $processor_data['processor_params']['key_file']);
        if (is_array($strResponse) && strlen($strResponse['error']) == 0) {
            $dataInit = $this->XmlToArray($strResponse['data']);
            if ($dataInit && $dataInit['Response']['Status'] == '00') {
                if ($dataInit['Response']['Order']['row']['Orderstatus'] == 'APPROVED') {
                    $pp_response = array(
                        'order_status' => 'P',
                        'pay_result' => 'OK',
                    );
                    fn_finish_payment($order_id, $pp_response);
                    fn_order_placement_routines('route', $order_id, false);
                    return;
                }
            }
        }

        $pp_response['order_status'] = 'F';

        $pp_response = array(
            'order_status' => 'F',
            'reason_text' => 'Error pay'
        );

        fn_finish_payment($order_id, $pp_response);
        fn_order_placement_routines('route', $order_id, false);
    }

    private function getPayUrl($test)
    {
        if ($test = 'test') {
            return 'https://mpitest.bspb.ru:5443/Exec';
        } else {
            return 'https://mpi.bspb.ru:5443/Exec';
        }
    }

    private function sendPostXml($url, $xml, $pathCertPem, $pathCertKey)
    {
        $path = $_SERVER['DOCUMENT_ROOT'] . '/var/bspb-keys/';

        $error = '';
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_RESOLVE, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSLCERT, $path . $pathCertPem);
        curl_setopt($ch, CURLOPT_SSLKEY, $path . $pathCertKey);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_VERBOSE, 1);
        if (!$answer = curl_exec($ch)) {
            $error = 'Error with connection: ' . curl_errno($ch);
        }
        curl_close($ch);

        return array(
            'data' => $answer,
            'error' => $error
        );
    }

    private function XmlToArray($strXml)
    {
        $strXml = str_replace("\\\"", "'", $strXml);
        $json = json_encode(simplexml_load_string($strXml));
        return json_decode($json, true);
    }

    private function getResultUrl()
    {
        return array(
            'ApproveURL' => fn_url('') . 'bspb-notification-callback',
            'CancelURL' => fn_url('') . 'bspb-notification-error',
            'DeclineURL' => fn_url('') . 'bspb-notification-error',
        );
    }

    private function findOrderByDesc($desc)
    {
        return str_replace('Опалат заказа №', '', $desc);
    }
}
